Package: MEDIPS
Type: Package
Title: DNA IP-seq data analysis
Version: 1.64.0
Date: 2020-02-15
Author: Lukas Chavez, Matthias Lienhard, Joern Dietrich, Isaac Lopez Moyado
Maintainer: Lukas Chavez <lukaschavez@ucsd.edu>
Description: MEDIPS was developed for analyzing data derived from
        methylated DNA immunoprecipitation (MeDIP) experiments followed
        by sequencing (MeDIP-seq). However, MEDIPS provides functionalities for the analysis of any kind of quantitative
        sequencing data (e.g. ChIP-seq, MBD-seq, CMS-seq and others)
        including calculation of differential coverage between groups
        of samples and saturation and correlation analysis.
License: GPL (>=2)
LazyLoad: yes
Depends: R (>= 3.0), BSgenome, Rsamtools
Imports: GenomicRanges, Biostrings, graphics, gtools, IRanges, methods,
        stats, utils, edgeR, DNAcopy, biomaRt, rtracklayer,
        preprocessCore
Suggests: BSgenome.Hsapiens.UCSC.hg19, MEDIPSData, BiocStyle
biocViews: DNAMethylation, CpGIsland, DifferentialExpression,
        Sequencing, ChIPSeq, Preprocessing, QualityControl,
        Visualization, Microarray, Genetics, Coverage,
        GenomeAnnotation, CopyNumberVariation, SequenceMatching
NeedsCompilation: no
Packaged: 2026-04-28 23:06:41 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/MEDIPS
git_branch: RELEASE_3_23
git_last_commit: 0101639
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
